@@ -3,10 +3,11 @@ orthogonalize <- function(X, group)
   n <- nrow(X)
   J <- max(group)
   Uinv <- vector("list", J)
-  XX <- matrix(NA, nrow=n, ncol=ncol(X))
+  XX <- X
   for (j in 1:J) {
     ind <- which(group==j)
-    U <- chol(crossprod(X[,ind])/n)
+    if (length(ind)==0) next
+    U <- chol(crossprod(X[, ind, drop=FALSE])/n)
     Uinv[[j]] <- backsolve(U,diag(length(ind)))
     XX[,ind] <- X[,ind] %*% Uinv[[j]]
   }
@@ -19,6 +20,7 @@ unorthogonalize <- function(b, XX, group)
   J <- max(group)
   for (j in 1:J) {
     ind <- which(group==j)
+    if (length(ind)==0) next
     beta[ind+1,] <- attr(XX,"Uinv")[[j]] %*% b[ind+1,]
   }
   beta
